import os

from typer import Argument, Option, Typer, echo

from snok import __version__
from snok.const import APP_DESC, APP_NAME, ProjectType
from snok.services.new import NewPackageService

app = Typer(
    name=APP_NAME,
    help=APP_DESC,
    no_args_is_help=True,
)


@app.callback()
def main_callback() -> None:
    pass


main_callback.__doc__ = APP_NAME


@app.command("version")
def _version() -> None:
    """Prints the version."""
    echo(__version__, nl=False)


@app.command(
    "new",
    help="Create various types of Python projects.",
    no_args_is_help=True,
)
def _new(
    name: str = Argument(
        ...,
        help="The name of the project.",
    ),
    description: str = Option(
        "A new Python project.",
        "-d",
        "--description",
        help="The description of the project. Defaults to 'A new Python project.'",
    ),
    output_dir: str = Option(
        os.getcwd(),
        "-o",
        "--output-dir",
        help="The output directory of the project. Defaults to the current directory.",
    ),
    type: ProjectType = Option(
        "package",
        "-t",
        "--type",
        help="The type of project to create. Defaults to 'package'",
    ),
) -> None:
    new_project_service_dispatcher = {
        ProjectType.package: NewPackageService,
    }
    new_project_service = new_project_service_dispatcher[type]()
    new_project_service.create(
        name=name,
        desc=description,
        type=type,
        output_dir=output_dir,
    )



from enum import Enum, unique

APP_NAME = "snok"
APP_DESC = "🚀 A simple, modern, full-stack toolkit for Python 🐍"


@unique
class ProjectType(str, Enum):
    package = "package"
    # app = "app" TODO: Add support



import os

from jinja2 import Template

from snok.const import ProjectType


class BaseNewService:
    def __init__(self) -> None:
        self.root_template_dir = "snok/templates"

    def create(
        self,
        name: str,
        desc: str,
        output_dir: str,
        type: ProjectType = ProjectType.package,
    ) -> None:
        raise NotImplementedError("create() must be implemented by subclass.")

    def _render_content_directory(
        self,
        name: str,
        desc: str,
        source_dir: str,
        output_dir: str,
        type: ProjectType = ProjectType.package,
    ) -> None:
        for root, dirs, files in os.walk(source_dir):
            if not ("__pycache__" in root or "_cache" in root):
                for file in files:
                    file_path = os.path.join(root, file)
                    relative_path = os.path.relpath(file_path, source_dir)
                    output_path = os.path.join(output_dir, relative_path)
                    if output_path.endswith("_pyproject.toml"):
                        output_path = output_path.replace(
                            "_pyproject.toml", "pyproject.toml"
                        )
                    self._render_content_file(
                        name=name,
                        desc=desc,
                        source_path=file_path,
                        output_path=output_path,
                        type=type,
                    )

    def _render_content_file(
        self,
        name: str,
        desc: str,
        source_path: str,
        output_path: str,
        type: ProjectType = ProjectType.package,
    ) -> None:
        with open(source_path, "r") as source_file:
            source_content = source_file.read()
            output_content = Template(source_content).render(
                __template_name=name,
                __template_description=desc,
            )
            with open(output_path, "w") as output_file:
                output_file.write(output_content)


class NewPackageService(BaseNewService):
    def __init__(self) -> None:
        super().__init__()

    def create(
        self,
        name: str,
        desc: str,
        output_dir: str,
        type: ProjectType = ProjectType.package,
    ) -> None:
        shared_template_dir = f"{self.root_template_dir}/__shared"
        package_template_dir = f"{self.root_template_dir}/__{type.value}"
        tests_template_dir = f"{self.root_template_dir}/__{type.value}_tests"
        root_output_dir = f"{output_dir}/{name}"
        os.makedirs(root_output_dir, exist_ok=True)
        self._render_content_directory(
            name=name,
            desc=desc,
            source_dir=shared_template_dir,
            output_dir=root_output_dir,
            type=type,
        )
        package_root = f"{root_output_dir}/{name}"
        os.makedirs(package_root, exist_ok=True)
        self._render_content_directory(
            name=name,
            desc=desc,
            source_dir=package_template_dir,
            output_dir=package_root,
            type=type,
        )
        package_test_root = f"{root_output_dir}/tests"
        os.makedirs(package_test_root, exist_ok=True)
        self._render_content_directory(
            name=name,
            desc=desc,
            source_dir=tests_template_dir,
            output_dir=package_test_root,
            type=type,
        )
        if not os.path.exists(f"{root_output_dir}/.git"):
            os.system(f"git init {root_output_dir} > /dev/null 2>&1")



import os

from typer.testing import CliRunner

from snok import __version__
from snok.cli import app


def test_new_package(cli_runner: CliRunner, setup_test_content_output_dir: str) -> None:
    response = cli_runner.invoke(
        app,
        ["new", "test_package", "-o", setup_test_content_output_dir],
    )
    assert response.exit_code == 0
    # assert the output directory exists
    for path in [
        f"{setup_test_content_output_dir}/test_package",
        f"{setup_test_content_output_dir}/test_package/test_package",
        f"{setup_test_content_output_dir}/test_package/tests",
        f"{setup_test_content_output_dir}/test_package/.git",
        f"{setup_test_content_output_dir}/test_package/README.md",
        f"{setup_test_content_output_dir}/test_package/setup.cfg",
        f"{setup_test_content_output_dir}/test_package/pyproject.toml",
        f"{setup_test_content_output_dir}/test_package/test_package/__init__.py",
        f"{setup_test_content_output_dir}/test_package/test_package/main.py",
        f"{setup_test_content_output_dir}/test_package/tests/__init__.py",
        f"{setup_test_content_output_dir}/test_package/tests/conftest.py",
        f"{setup_test_content_output_dir}/test_package/tests/test_main.py",
        f"{setup_test_content_output_dir}/test_package/tests/test_version.py",
    ]:
        assert os.path.exists(path)


def test_version(cli_runner: CliRunner) -> None:
    response = cli_runner.invoke(app, ["version"])
    assert response.exit_code == 0
    assert response.stdout == __version__
